require "logging"
require "bunny"
require "securerandom"

module ActiveMessage
  autoload :Consumer,       'active_message/consumer'
  autoload :Worker,         'active_message/worker'
  autoload :DemoWorker,     'active_message/demo_worker'
  autoload :DemoWorkerB,    'active_message/demo_worker_b'
  autoload :Broker,         'active_message/broker'
  autoload :Version,        'active_message/version'
  autoload :Logger,         'active_message/logger'
  autoload :Configuration,  'active_message/configuration'

  def self.register_consumer(klass)
    ActiveMessage::Logger.debug "Registered Consumer: #{klass.name}"
    self.consumers << klass
  end

  def self.consumers
    @consumers ||= []
  end

  def self.connect(config = ActiveMessage::Configuration)
    ActiveMessage::Logger.debug "Connecting to AMQP ...."
    @broker               = ActiveMessage::Broker.instance
    @broker.connect(config)
    @connected            = true
    ActiveMessage::Logger.debug "Connected to AMQP ...."
  end

  def self.broker
    @broker
  end

  def self.connected
    @connected
  end

  def self.publish(*args)
    self.broker.publish(*args)
  end

  def self.version
    ActiveMessage::VERSION
  end
end
