module ActiveMessage
  class Broker
    include Singleton

    attr_accessor :configuration, :connection, :exchange

    def initialize
    end

    def connect(config)
      @configuration  = config
      @connection     = Bunny.new(host: ActiveMessage::Configuration.get(:host), auth_mechanism: "EXTERNAL", tls: ActiveMessage::Configuration.get(:tls), tls_cert: ActiveMessage::Configuration.get(:tls_cert), tls_key: ActiveMessage::Configuration.get(:tls_key), tls_ca_certificates: ActiveMessage::Configuration.get(:tls_ca_certificates))
      @connection.start
      @exchange       = @connection.topic(@configuration[:exchange] || "demo", auto_delete: true)
    end

    def disconnect
      self.connection.close
    end

    def connection
      @connection
    end

    def exchange
      @exchange
    end

    def ack(channel, delivery_tag)
      channel.ack(delivery_tag, false)
    end

    def nack(channel, delivery_tag)
      channel.nack(delivery_tag, false, false)
    end

    def publish(routing_key, message, properties = {})
      ActiveMessage::Logger.debug "Publishing Message => '#{message.inspect}' to #{routing_key}"
      non_overridable_properties = {
        routing_key: routing_key,
        timestamp: Time.now.to_i,
        content_type: 'application/json'
      }
      properties[:message_id] ||= generate_id
      exchange.publish(message, { persistent: true }.merge(properties).merge(non_overridable_properties))
    end

    def generate_id
      SecureRandom.uuid
    end
  end
end