module ActiveMessage
  class UnknownAttributeError < StandardError; end

  module Configuration
    def self.initialize
      @config = {
        host: 'localhost',
        port: 5672,
        exchange: 'ActiveMessage',
        vhost: '/',
        tls: false,
        tls_cert: nil,
        tls_key: nil,
        tls_ca_certificates: [],
        username: 'dustin',
        password: ''
      }
    end

    def self.get(attr)
      check_attr(attr)
      user_config[attr]
    end

    def self.set(attr, value)
      check_attr(attr)
      user_config[attr] = value
    end

    class << self
      alias_method :[],  :get
      alias_method :[]=, :set
    end

    def self.user_config
      initialize unless @config
      @config
    end

    def self.check_attr(attr)
      unless user_config.key?(attr)
        raise ActiveMessage::UnknownAttributeError, "#{attr} is not a valid config attribute"
      end
    end

    def self.load_from_file(file, environment = nil)
      active_env = (environment.nil?) ? Rails.env : environment
      temp = YAML.load(File.read(file))[active_env]
      temp.each do |attr, value|
        ActiveMessage::Configuration.send("#{attr}=", value)
      end
    end

    def self.method_missing(method, *args, &block)
      attr = method.to_s.sub(/=$/, '').to_sym
      return super unless user_config.key?(attr)

      if method =~ /=$/
        set(attr, args.first)
      else
        get(attr)
      end
    end
  end
end