module ActiveMessage
  module Consumer
    def self.included(base)
      base.extend(ClassMethods)
      ActiveMessage.register_consumer(base)
    end

    module ClassMethods
      def consume(*routing_keys)
        ActiveMessage::Logger.debug "Consumer is consuming #{routing_keys.join(", ")} ...."
        @routing_keys = self.routing_keys.union(routing_keys)
      end

      def queue_name(name)
        @queue_name = name
      end

      def get_queue_name
        return @queue_name unless @queue_name.nil?
        queue_name = self.name.gsub(/::/, ':')
        queue_name.gsub!(/([^A-Z:])([A-Z])/) { "#{$1}_#{$2}" }
        queue_name.downcase
      end

      def routing_keys
        @routing_keys ||= Set.new
      end

      def process(delivery_info, properties, payload)
        ActiveMessage::Logger.debug "Please implement this method in your consumer ...."
      end
    end
  end
end