module ActiveMessage
  class Logger
    include Singleton

    def initialize
      setup_colorization
      Logging.appenders.stdout(
        'stdout',
        layout: Logging.layouts.pattern(
          pattern: '[%d] %-5l %c: %m\n',
          color_scheme: 'bright'
        )
      )
      @app_log = Logging.logger['ActiveMessage']
      @app_log.add_appenders 'stdout'
      @app_log.level = :debug
    end

    def log
      @app_log
    end

    def self.debug(message)
      self.instance.log.debug(message)
    end

    def self.info(message)
      self.instance.log.info(message)
    end

    def self.warn(message)
      self.instance.log.warn(message)
    end

    def self.error(message)
      self.instance.log.error(message)
    end

    def self.fatal(message)
      self.instance.log.fatal(message)
    end

    private
    def setup_colorization
      Logging.color_scheme('bright',
        levels: {
          info:   :green,
          warn:   :yellow,
          error:  :red,
          fatal:  [:white, :on_red]
        },
        date:     :blue,
        logger:   :cyan,
        message:  :magenta
      )
    end
  end
end